package com.heyang;

import java.util.List;

public class Evaluator {
	private List<Token> tokens;
	private int index;
	
	public Evaluator(List<Token> tokens) {
		this.tokens=tokens;
		index=0;
	}
	
	public double evaluate() throws AnalyzerExp{
		return parse_expression();
	}
	
	private double parse_expression() throws AnalyzerExp{
		double left,right;
		Token currentToken;
		
		left=parse_term();
		for(;;) {
			if(index==tokens.size()) {
				return left;
			}else {
				currentToken=tokens.get(index);
				index++;
			}
			
			if(currentToken.getType()!=Token.TYPE_PLUS && currentToken.getType()!=Token.TYPE_MINUS) {
				index--;
				break;
			}
			
			right=parse_term();
			
			if(currentToken.getType()==Token.TYPE_PLUS) {
				left+= right;
			}else if(currentToken.getType()==Token.TYPE_MINUS) {
				left-= right;
			}else {
				index--;
			}
		}
		
		return left;
	}
	
	private double parse_term() throws AnalyzerExp{
		double left,right;
		Token currentToken;
		
		left=parse_primary_exp();
		for(;;) {
			if(index==tokens.size()) {
				return left;
			}else {
				currentToken=tokens.get(index);
				index++;
			}
			
			if(currentToken.getType()!=Token.TYPE_MULTI && currentToken.getType()!=Token.TYPE_DIVIDE) {
				index--;
				break;
			}
			
			right=parse_primary_exp();
			
			if(currentToken.getType()==Token.TYPE_MULTI) {
				left*= right;
			}else if(currentToken.getType()==Token.TYPE_DIVIDE) {
				left/= right;
			}
		}
		
		return left;
	}
	
	private double parse_primary_exp() throws AnalyzerExp{
		Token token;
		double retal=0.0;

		if(index==tokens.size()) {
			return retal;
		}else {
			token=tokens.get(index);
			index++;
		}
		
		if(token.getType()==Token.TYPE_DIGITAL) {
			retal= token.getValue();
			return retal;
		}else if(token.getType()==Token.TYPE_LEFT_PAREN){
			retal=parse_expression();
			
			if(index==tokens.size()) {
				return retal;
			}else {
				token=tokens.get(index);
				index++;
			}
			
			if(token.getType()!=Token.TYPE_RIGHT_PAREN) {
				throw new AnalyzerExp("missing )");
			}
			
			return retal;
		}else {
			throw new AnalyzerExp(token+" should be a digital.");
		}
	}
	
	public static void main(String[] args) throws Exception{
		String exp="1+(2*3-4)*5-6";
		System.out.println("exp="+exp);
		Lexer lexer=new Lexer(exp);
		Evaluator e=new Evaluator(lexer.getTokens());
		System.out.println(e.evaluate());
	}
}
